@@ -47,8 +47,9 @@ OHProduct::OHProduct(UpMpd *dev, const string& friendlyname, bool hasRcv)
     : UpnpService(sTpProduct, sIdProduct, dev), m_dev(dev),
       m_roomOrName(friendlyname), m_sourceIndex(0)
 {
+    // Playlist must stay first.
     o_sources.push_back("Playlist");
-    o_sources.push_back("UpnpAv");
+    //o_sources.push_back("UpnpAv");
     if (hasRcv) 
         o_sources.push_back("Receiver");
 
@@ -171,7 +172,7 @@ int OHProduct::standby(const SoapIncoming& sc, SoapOutgoing& data)
 int OHProduct::setStandby(const SoapIncoming& sc, SoapOutgoing& data)
 {
     LOGDEB("OHProduct::setStandby" << endl);
-    if (!sc.getBool("Value", &m_standby)) {
+    if (!sc.get("Value", &m_standby)) {
         return UPNP_E_INVALID_PARAM;
     }
     m_dev->loopWakeup();
@@ -199,14 +200,9 @@ int OHProduct::sourceIndex(const SoapIncoming& sc, SoapOutgoing& data)
     return UPNP_E_SUCCESS;
 }
 
-int OHProduct::setSourceIndex(const SoapIncoming& sc, SoapOutgoing& data)
+int OHProduct::iSetSourceIndex(int sindex)
 {
-    LOGDEB("OHProduct::setSourceIndex" << endl);
-    int sindex;
-    if (!sc.getInt("Value", &sindex)) {
-        return UPNP_E_INVALID_PARAM;
-    }
-    LOGDEB("OHProduct::setSourceIndex: " << sindex << endl);
+    LOGDEB("OHProduct::iSetSourceIndex: " << sindex << endl);
     if (sindex < 0 || sindex >= int(o_sources.size())) {
         LOGERR("OHProduct::setSourceIndex: bad index: " << sindex << endl);
         return UPNP_E_INVALID_PARAM;
@@ -215,20 +211,28 @@ int OHProduct::setSourceIndex(const SoapIncoming& sc, SoapOutgoing& data)
     m_dev->loopWakeup();
     return UPNP_E_SUCCESS;
 }
+int OHProduct::setSourceIndex(const SoapIncoming& sc, SoapOutgoing&)
+{
+    LOGDEB("OHProduct::setSourceIndex" << endl);
+    int sindex;
+    if (!sc.get("Value", &sindex)) {
+        return UPNP_E_INVALID_PARAM;
+    }
+    return iSetSourceIndex(sindex);
+}
 
 int OHProduct::setSourceIndexByName(const SoapIncoming& sc, SoapOutgoing& data)
 {
-    LOGDEB("OHProduct::setSourceIndexByName" << endl);
 
     string name;
-    if (!sc.getString("Value", &name)) {
+    if (!sc.get("Value", &name)) {
+        LOGERR("OHProduct::setSourceIndexByName: no Value" << endl);
         return UPNP_E_INVALID_PARAM;
     }
+    LOGDEB("OHProduct::setSourceIndexByName: " << name << endl);
     for (unsigned int i = 0; i < o_sources.size(); i++) {
         if (o_sources[i] == name) {
-            m_sourceIndex = i;
-            m_dev->loopWakeup();
-            return UPNP_E_SUCCESS;
+            return iSetSourceIndex(i);
         }
     }
             
@@ -240,7 +244,7 @@ int OHProduct::source(const SoapIncoming& sc, SoapOutgoing& data)
 {
     LOGDEB("OHProduct::source" << endl);
     int sindex;
-    if (!sc.getInt("Index", &sindex)) {
+    if (!sc.get("Index", &sindex)) {
         return UPNP_E_INVALID_PARAM;
     }
     LOGDEB("OHProduct::source: " << sindex << endl);
