@@ -25,6 +25,8 @@
 #include <stdlib.h>                     // for atoi, getenv, exit
 #include <sys/param.h>                  // for MIN
 #include <unistd.h>                     // for geteuid, chown, sleep, etc
+#include <string.h>                     // for memset
+#include <grp.h>
 
 #include <iostream>                     // for basic_ostream, operator<<, etc
 #include <string>                       // for string, operator<<, etc
@@ -47,6 +49,7 @@
 #include "renderctl.hxx"                // for UpMpdRenderCtl
 #include "upmpdutils.hxx"               // for path_cat, Pidfile, regsub1, etc
 #include "execmd.h"
+#include "httpfs.hxx"
 
 using namespace std;
 using namespace std::placeholders;
@@ -54,18 +57,6 @@ using namespace UPnPP;
 
 // Note: if we ever need this to work without cxx11, there is this:
 // http://www.tutok.sk/fastgl/callback.html
-//
-// Note that there is a problem in the order in which we do things
-// here: because the UpnpDevice() constructor starts the
-// advertisements and publishes the description document before the
-// services are actually initialized, it is possible that a fast
-// client will fail in subscribing to events, which will manifest
-// itself on the server side by error messages like the following in
-// the log:
-//   libupnpp/device/device.cxx:183::UpnpDevice: Bad serviceID: 
-//        urn:upnp-org:serviceId:ConnectionManager
-// The solution would be to have a separate init call to start the
-// device at the end of the constructor code.
 UpMpd::UpMpd(const string& deviceid, const string& friendlyname,
              const unordered_map<string, VDirContent>& files,
              MPDCli *mpdcli, Options opts)
@@ -83,16 +74,29 @@ UpMpd::UpMpd(const string& deviceid, const string& friendlyname,
     m_services.push_back(new UpMpdConMan(this));
     bool ohReceiver = (m_options & upmpdOhReceiver) != 0; 
     if (m_options & upmpdDoOH) {
-        m_services.push_back(new OHProduct(this, friendlyname, ohReceiver));
+        OHProduct *ohpr = new OHProduct(this, friendlyname, ohReceiver);
+        m_services.push_back(ohpr);
         m_services.push_back(new OHInfo(this));
         m_services.push_back(new OHTime(this));
         m_services.push_back(new OHVolume(this, rdctl));
-        OHPlaylist *ohp = new OHPlaylist(this, rdctl, opts.ohmetasleep);
-        m_services.push_back(ohp);
+        OHPlaylist *ohpl = new OHPlaylist(this, rdctl, opts.ohmetasleep);
+        m_services.push_back(ohpl);
         if (avt)
-            avt->setOHP(ohp);
+            avt->setOHP(ohpl);
         if (ohReceiver) {
-            m_services.push_back(new OHReceiver(this, ohp, opts.schttpport));
+            struct OHReceiverParams parms;
+            parms.pl = ohpl;
+            parms.pr = ohpr;
+            if (opts.schttpport)
+                parms.httpport = opts.schttpport;
+            if (!opts.scplaymethod.empty()) {
+                if (!opts.scplaymethod.compare("alsa")) {
+                    parms.pm = OHReceiverParams::OHRP_ALSA;
+                } else if (!opts.scplaymethod.compare("mpd")) {
+                    parms.pm = OHReceiverParams::OHRP_MPD;
+                }
+            }
+            m_services.push_back(new OHReceiver(this, parms));
         }
     }
 }
@@ -154,87 +158,10 @@ Usage(void)
     exit(1);
 }
 
-// The description XML document is the first thing downloaded by
-// clients and tells them what services we export, and where to find
-// them.
-static string ohDesc(
-    "<service>"
-    "  <serviceType>urn:av-openhome-org:service:Product:1</serviceType>"
-    "  <serviceId>urn:av-openhome-org:serviceId:Product</serviceId>"
-    "  <SCPDURL>/OHProduct.xml</SCPDURL>"
-    "  <controlURL>/ctl/OHProduct</controlURL>"
-    "  <eventSubURL>/evt/OHProduct</eventSubURL>"
-    "</service>"
-    "<service>"
-    "  <serviceType>urn:av-openhome-org:service:Info:1</serviceType>"
-    "  <serviceId>urn:av-openhome-org:serviceId:Info</serviceId>"
-    "  <SCPDURL>/OHInfo.xml</SCPDURL>"
-    "  <controlURL>/ctl/OHInfo</controlURL>"
-    "  <eventSubURL>/evt/OHInfo</eventSubURL>"
-    "</service>"
-    "<service>"
-    "  <serviceType>urn:av-openhome-org:service:Time:1</serviceType>"
-    "  <serviceId>urn:av-openhome-org:serviceId:Time</serviceId>"
-    "  <SCPDURL>/OHTime.xml</SCPDURL>"
-    "  <controlURL>/ctl/OHTime</controlURL>"
-    "  <eventSubURL>/evt/OHTime</eventSubURL>"
-    "</service>"
-    "<service>"
-    "  <serviceType>urn:av-openhome-org:service:Volume:1</serviceType>"
-    "  <serviceId>urn:av-openhome-org:serviceId:Volume</serviceId>"
-    "  <SCPDURL>/OHVolume.xml</SCPDURL>"
-    "  <controlURL>/ctl/OHVolume</controlURL>"
-    "  <eventSubURL>/evt/OHVolume</eventSubURL>"
-    "</service>"
-    "<service>"
-    "  <serviceType>urn:av-openhome-org:service:Playlist:1</serviceType>"
-    "  <serviceId>urn:av-openhome-org:serviceId:Playlist</serviceId>"
-    "  <SCPDURL>/OHPlaylist.xml</SCPDURL>"
-    "  <controlURL>/ctl/OHPlaylist</controlURL>"
-    "  <eventSubURL>/evt/OHPlaylist</eventSubURL>"
-    "</service>"
-    );
-
-// We only advertise the Openhome Receiver service if the sc2mpd
-// songcast-to-mpd gateway command is available
-static string ohDescReceive(
-    "<service>"
-    "  <serviceType>urn:av-openhome-org:service:Receiver:1</serviceType>"
-    "  <serviceId>urn:av-openhome-org:serviceId:Receiver</serviceId>"
-    "  <SCPDURL>/OHReceiver.xml</SCPDURL>"
-    "  <controlURL>/ctl/OHReceiver</controlURL>"
-    "  <eventSubURL>/evt/OHReceiver</eventSubURL>"
-    "</service>"
-    );
-
-static const string iconDesc(
-    "<iconList>"
-    "  <icon>"
-    "    <mimetype>image/png</mimetype>"
-    "    <width>64</width>"
-    "    <height>64</height>"
-    "    <depth>32</depth>"
-    "    <url>/icon.png</url>"
-    "  </icon>"
-    "</iconList>"
-    );
-
-// Our XML description data. !Keep description.xml first!
-static vector<const char *> xmlfilenames = 
-{
-    /* keep first */ "description.xml", /* keep first */
-    "RenderingControl.xml", "AVTransport.xml", "ConnectionManager.xml",
-};
-static vector<const char *> ohxmlfilenames = 
-{
-    "OHProduct.xml", "OHInfo.xml", "OHTime.xml", "OHVolume.xml", 
-    "OHPlaylist.xml",
-};
-
 static const string dfltFriendlyName("UpMpd");
 
 // This is global
-string upmpdProtocolInfo;
+string g_protocolInfo;
 
 // Static for cleanup in sig handler.
 static UpnpDevice *dev;
@@ -281,6 +208,7 @@ int main(int argc, char *argv[])
     bool ohmetapersist = true;
     string upmpdcliuser("upmpdcli");
     string pidfilename("/var/run/upmpdcli.pid");
+    string presentationhtml(DATADIR "/presentation.html");
     string iface;
     unsigned short upport = 0;
     string upnpip;
@@ -346,6 +274,7 @@ int main(int argc, char *argv[])
     UpMpd::Options opts;
 
     string iconpath;
+    string cachedir;
     if (!g_configfilename.empty()) {
         ConfSimple config(g_configfilename.c_str(), 1, true);
         if (!config.ok()) {
@@ -375,6 +304,8 @@ int main(int argc, char *argv[])
             ohmetapersist = atoi(value.c_str()) != 0;
         }
         config.get("iconpath", iconpath);
+        config.get("presentationhtml", presentationhtml);
+        config.get("cachedir", cachedir);
         if (!(op_flags & OPT_i)) {
             config.get("upnpiface", iface);
             if (iface.empty()) {
@@ -386,6 +317,7 @@ int main(int argc, char *argv[])
         }
         if (config.get("schttpport", value))
             opts.schttpport = atoi(value.c_str());
+        config.get("scplaymethod", opts.scplaymethod);
         config.get("sc2mpd", g_sc2mpd_path);
         if (config.get("ohmetasleep", value))
             opts.ohmetasleep = atoi(value.c_str());
@@ -400,20 +332,13 @@ int main(int argc, char *argv[])
         // Do we have an sc2mpd command installed (for songcast)?
         if (!ExecCmd::which("sc2mpd", g_sc2mpd_path))
             g_sc2mpd_path.clear();
-    } else {
-        if (access(g_sc2mpd_path.c_str(), X_OK|R_OK) != 0) {
-            LOGERR("Specified path for sc2mpd: " << g_sc2mpd_path << 
-                   " is not executable" << endl);
-            g_sc2mpd_path.clear();
-        }
     }
 
     Pidfile pidfile(pidfilename);
 
-    string cachedir;
-
     // If started by root, do the pidfile + change uid thing
     uid_t runas(0);
+    gid_t runasg(0);
     if (geteuid() == 0) {
         struct passwd *pass = getpwnam(upmpdcliuser.c_str());
         if (pass == 0) {
@@ -422,6 +347,7 @@ int main(int argc, char *argv[])
             return 1;
         }
         runas = pass->pw_uid;
+        runasg = pass->pw_gid;
 
         pid_t pid;
         if ((pid = pidfile.open()) != 0) {
@@ -433,9 +359,11 @@ int main(int argc, char *argv[])
             LOGFAT("Can't write pidfile: " << pidfile.getreason() << endl);
             return 1;
         }
-        cachedir = "/var/cache/upmpdcli";
+	if (cachedir.empty())
+            cachedir = "/var/cache/upmpdcli";
     } else {
-        cachedir = path_cat(path_tildexpand("~") , "/.cache/upmpdcli");
+	if (cachedir.empty())
+            cachedir = path_cat(path_tildexpand("~") , "/.cache/upmpdcli");
     }
 
     string& mcfn = opts.cachefn;
@@ -474,11 +402,33 @@ int main(int argc, char *argv[])
                 LOGERR("chown("<<logfilename<<") : errno : " << errno << endl);
             }
         }
+        if (initgroups(upmpdcliuser.c_str(), runasg) < 0) {
+            LOGERR("initgroup failed. Errno: " << errno << endl);
+        }
         if (setuid(runas) < 0) {
             LOGFAT("Can't set my uid to " << runas << " current: " << geteuid()
                    << endl);
             return 1;
         }
+#if 0        
+        gid_t list[100];
+        int ng = getgroups(100, list);
+        cerr << "GROUPS: ";
+        for (int i = 0; i < ng; i++) {
+            cerr << int(list[i]) << " ";
+        }
+        cerr << endl;
+#endif
+    }
+
+//// Dropped root 
+
+    if (!g_sc2mpd_path.empty()) {
+        if (access(g_sc2mpd_path.c_str(), X_OK|R_OK) != 0) {
+            LOGERR("Specified path for sc2mpd: " << g_sc2mpd_path << 
+                   " is not executable" << endl);
+            g_sc2mpd_path.clear();
+        }
     }
 
     // Initialize MPD client object. Retry until it works or power fail.
@@ -540,64 +490,12 @@ int main(int argc, char *argv[])
     // Create unique ID
     string UUID = LibUPnP::makeDevUUID(friendlyname, hwaddr);
 
-    // Read our XML data to make it available from the virtual directory
-    if (openhome) {
-        if (!g_sc2mpd_path.empty()) {
-            ohxmlfilenames.push_back("OHReceiver.xml");
-        }
-        xmlfilenames.insert(xmlfilenames.end(), ohxmlfilenames.begin(),
-                            ohxmlfilenames.end());
-    }
-
-    {
-        string protofile = path_cat(datadir, "protocolinfo.txt");
-        if (!read_protocolinfo(protofile, upmpdProtocolInfo)) {
-            LOGFAT("Failed reading protocol info from " << protofile << endl);
-            return 1;
-        }
-    }
-
-    string reason;
-
-    string icondata;
-    if (!iconpath.empty()) {
-        if (!file_to_string(iconpath, icondata, &reason)) {
-            LOGERR("Failed reading " << iconpath << " : " << reason << endl);
-        }
-    }
-
+    // Initialize the data we serve through HTTP (device and service
+    // descriptions, icons, presentation page, etc.)
     unordered_map<string, VDirContent> files;
-    for (unsigned int i = 0; i < xmlfilenames.size(); i++) {
-        string filename = path_cat(datadir, xmlfilenames[i]);
-        string data;
-        if (!file_to_string(filename, data, &reason)) {
-            LOGFAT("Failed reading " << filename << " : " << reason << endl);
-            return 1;
-        }
-        if (i == 0) {
-            // Special for description: set UUID and friendlyname
-            data = regsub1("@UUID@", data, UUID);
-            data = regsub1("@FRIENDLYNAME@", data, friendlyname);
-            if (openhome) {
-                if (!g_sc2mpd_path.empty()) {
-                    ohDesc += ohDescReceive;
-                }
-                data = regsub1("@OPENHOME@", data, ohDesc);
-            } else {
-                data = regsub1("@OPENHOME@", data, "");
-            }
-            if (!icondata.empty())
-                data = regsub1("@ICONLIST@", data, iconDesc);
-            else
-                data = regsub1("@ICONLIST@", data, "");
-        }
-        files.insert(pair<string, VDirContent>
-                     (xmlfilenames[i], VDirContent(data, "application/xml")));
-    }
-
-    if (!icondata.empty()) {
-        files.insert(pair<string, VDirContent>
-                     ("icon.png", VDirContent(icondata, "image/png")));
+    if (!initHttpFs(files, datadir, UUID, friendlyname, openhome, iconpath,
+            presentationhtml)) {
+        exit(1);
     }
 
     if (ownqueue)
